#!/bin/ksh
#**********************************************************************#
#*                                                                    *#
#* Copyright (c) 2002 by Sun Microsystems, Inc.                       *#
#* All rights reserved.                                               *#
#*                                                                    *#
#**********************************************************************#

# Sun Cluster Data Services Builder template version 1.0
#
# Start Method for scmtp.
#
# The data service is started under the control of PMF. Prior to starting the 
# process(es) for scmtp some sanity checks are done. The PMF tries to start the
# service specified number of times and if the number of attempts exceeds this 
# value within a specified interval of time the PMF reports a failure to start
# the service. The number of times to retry and the interval in which it is to
# be tried are both properties of the resource set in RTR file. 

###############################################################################
# Parse program arguments.
#
function parse_args # [args ...]
{
	typeset opt

	while getopts 'R:G:T:' opt
	do
		case "$opt" in

		R)
		# Name of the scmtp resource.
		RESOURCE_NAME=$OPTARG
		;;

		G)
		# Name of the resource group in which the resource is
		# configured.
		RESOURCEGROUP_NAME=$OPTARG
		;;

		T)
		# Name of the resource type.
		RESOURCETYPE_NAME=$OPTARG
		;;

		*)
		logger -p ${SYSLOG_FACILITY}.err -t \
		[$RESOURCETYPE_NAME,$RESOURCEGROUP_NAME,$RESOURCE_NAME] \
		"ERROR: Option $OPTARG unknown"
		exit 1
		;;

		esac
	done

}

###############################################################################
# MAIN
##############################################################################
DEBUG_PARMS="$*"

export PATH=/bin:/usr/bin:/usr/cluster/bin:/usr/sbin:/usr/proc/bin:$PATH

# Obtain the syslog facility to use. This will be used to log the messages.
SYSLOG_FACILITY=`scha_cluster_get -O SYSLOG_FACILITY`

# Parse the arguments that have been passed to this method
parse_args "$@"

PMF_TAG=$RESOURCEGROUP_NAME,$RESOURCE_NAME,0.svc
SYSLOG_TAG=$RESOURCETYPE_NAME,$RESOURCEGROUP_NAME,$RESOURCE_NAME

emp_debug_info="`scha_resource_get -O Extension -R $RESOURCE_NAME \
        -G $RESOURCEGROUP_NAME Debug`"
EMP_DEBUG=`echo $emp_debug_info | awk '{print $2}'`

# We need to know the full path for the gethostnames utility which resides
# in the directory <RT_BASEDIR>. Get this from the RT_BASEDIR property of the
# resource type.
RT_BASEDIR=`scha_resource_get -O RT_BASEDIR -R $RESOURCE_NAME \
	-G $RESOURCEGROUP_NAME`

emp_setup_info="`scha_resource_get -O Extension -R $RESOURCE_NAME \
	-G $RESOURCEGROUP_NAME EMP_SETUP`"
EMP_SETUP=`echo $emp_setup_info | awk '{print $2}'`

# We need to know the full path for the mtp utilities which reside in the
# directory <EMP_BASEDIR>. Get this from the EMP_BASEDIR property of the
# resource type.
emp_basedir_info="`scha_resource_get -O Extension -R $RESOURCE_NAME \
	-G $RESOURCEGROUP_NAME EMP_BASEDIR`"
EMP_BASEDIR=`echo $emp_basedir_info | awk '{print $2}'`

# We need to know the mtp administrator name to start the mtp utilities
# with his/her user id. Get this from the EMP_ADMIN property of the
# resource type.
emp_admin_info="`scha_resource_get -O Extension -R $RESOURCE_NAME \
	-G $RESOURCEGROUP_NAME EMP_ADMIN`"
EMP_ADMIN=`echo $emp_admin_info | awk '{print $2}'`

# We need to know the monitoring level.
emp_monitor_info="`scha_resource_get -O Extension -R $RESOURCE_NAME \
	-G $RESOURCEGROUP_NAME Monitor`"
EMP_MONITOR=`echo $emp_monitor_info | awk '{print $2}'`

emp_child_info="`scha_resource_get -O Extension -R $RESOURCE_NAME \
	-G $RESOURCEGROUP_NAME Child_mon_level`"
EMP_CHILD_MON_LEVEL=`echo $emp_child_info | awk '{print $2}'`

if [ $EMP_CHILD_MON_LEVEL -eq -1 ]
then
  CHILD_LEVEL=" "
else
  CHILD_LEVEL="-C $EMP_CHILD_MON_LEVEL"
fi

# get the Timeout value allowed for start method from the RTR file
START_TIMEOUT=`scha_resource_get -O START_TIMEOUT -R $RESOURCE_NAME \
	-G $RESOURCEGROUP_NAME`
((WAIT_START_TIMEOUT=$START_TIMEOUT * 5/100))

hostnames=`$RT_BASEDIR/gethostnames -R $RESOURCE_NAME -G $RESOURCEGROUP_NAME \
	-T $RESOURCETYPE_NAME`

EMP_RESOURCE_ID="type $RESOURCETYPE_NAME, group $RESOURCEGROUP_NAME, name $RESOURCE_NAME, host $hostnames"; export EMP_RESOURCE_ID

start_cmd_args="$EMP_BASEDIR/scbin/mtp_start -S $EMP_SETUP -D $EMP_DEBUG -I \"$EMP_RESOURCE_ID\" -L \"$SYSLOG_TAG\" -F \"$SYSLOG_FACILITY\" > /dev/null 2>&1"

start_cmd_prog=`echo $start_cmd_args | nawk '{print $1}'`

if [[ ! -f $start_cmd_prog || ! -x $start_cmd_prog ]]; then
	logger -p ${SYSLOG_FACILITY}.err -t [$SYSLOG_TAG] \
		"${ARGV0} File $start_cmd_prog is missing or not executable"
	exit 1
fi

# Get the value for retry count from the RTR file.
RETRY_CNT=`scha_resource_get -O Retry_Count -R $RESOURCE_NAME \
	-G $RESOURCEGROUP_NAME`

# Get the value for retry interval from the RTR file. The value for the 
# RETRY_INTERVAL in the RTR file will be in seconds. Convert this value from 
# seconds to minutes for passing on to pmfadm. Note that this is necessarily
# a conversion with round-up, eg. 59 seconds --> 1 minute.
((RETRY_INTRVAL = (`scha_resource_get -O Retry_Interval -R $RESOURCE_NAME -G \
	$RESOURCEGROUP_NAME` + 59) / 60 ))

if [ "$EMP_DEBUG" = "ON" ]
then
	logger -p ${SYSLOG_FACILITY}.notice \
		-t [$SYSLOG_TAG] \
		"${ARGV0} Starting start method for scmtp resource with parameters $DEBUG_PARMS"
fi
# User added code -- BEGIN vvvvvvvvvvvvvvv
# User added code -- END   ^^^^^^^^^^^^^^^

if [ "$EMP_MONITOR" = "NONE" -o "$EMP_MONITOR" = "none" ]
then
# start the daemon under the control of Sun Cluster Process Monitor
# Facility only once.
  if [ "$EMP_DEBUG" = "ON" ]
  then
	logger -p ${SYSLOG_FACILITY}.notice \
		-t [$SYSLOG_TAG] \
		"${ARGV0} Start method executes pmfadm -c $PMF_TAG -n 0 $CHILD_LEVEL su $EMP_ADMIN -c \"$start_cmd_args\""
  fi
  pmfadm -c $PMF_TAG -n 0 $CHILD_LEVEL su $EMP_ADMIN -c "$start_cmd_args" >/dev/null 2>&1

else
# start the daemon under the control of Sun Cluster Process Monitor
# Facility. Let it crash and restart up to $RETRY_COUNT times in a period of 
# $RETRY_INTERVAL; if it crashes more often than that, the process monitor
# facility will cease trying to restart it
  if [ "$EMP_DEBUG" = "ON" ]
  then
	logger -p ${SYSLOG_FACILITY}.notice \
		-t [$SYSLOG_TAG] \
		"${ARGV0} Start method executes pmfadm -c $PMF_TAG -n $RETRY_CNT -t $RETRY_INTRVAL $CHILD_LEVEL su $EMP_ADMIN -c \"$start_cmd_args\""
  fi
  pmfadm -c $PMF_TAG -n $RETRY_CNT -t $RETRY_INTRVAL $CHILD_LEVEL su $EMP_ADMIN -c "$start_cmd_args" >/dev/null 2>&1

fi

if [ "$EMP_DEBUG" = "ON" ]
then
	logger -p ${SYSLOG_FACILITY}.notice \
		-t [$SYSLOG_TAG] \
		"${ARGV0} Start method waiting $WAIT_START_TIMEOUT"
fi
sleep $WAIT_START_TIMEOUT
pmfadm -q $PMF_TAG >/dev/null 2>&1
pmfadm_status=$?
if [ "$EMP_DEBUG" = "ON" ]
then
	logger -p ${SYSLOG_FACILITY}.notice \
		-t [$SYSLOG_TAG] \
		"${ARGV0} Start method \"pmfadm -q $PMF_TAG\" returns $pmfadm_status"
fi

# User added code -- BEGIN vvvvvvvvvvvvvvv
# User added code -- END   ^^^^^^^^^^^^^^^

if [ "$EMP_DEBUG" = "ON" ]
then
	logger -p ${SYSLOG_FACILITY}.notice \
		-t [$SYSLOG_TAG] \
		"${ARGV0} Start method exit with status 0"
fi
exit 0
